import array
from sdk32 import *

ps = PAINTSTRUCT()
rect = RECT()
tm = TEXTMETRIC()
lines = []
xchar = ychar = xclient = yclient = xbuffer = ybuffer = 0
xcaret = ycaret = 0

def wnd_proc(wnd, msg, wparam, lparam):
    global ps, rect, tm, lines
    global xchar, ychar, xclient, yclient, xbuffer, ybuffer
    global xcaret, ycaret
    if msg == WM_CREATE:
        dc = GetDC(wnd)
        SelectObject(dc, GetStockObject(SYSTEM_FIXED_FONT))
        GetTextMetrics(dc, tm)
        xchar = tm.ave_char_width
        ychar = tm.height
        ReleaseDC(wnd, dc)
        return 0

    elif msg == WM_SIZE:
        xclient = LOWORD(lparam)
        yclient = HIWORD(lparam)
        xbuffer = max(1, xclient / xchar)
        ybuffer = max(1, yclient / ychar)
        lines = []
        for i in range(ybuffer):
            lines.append(array.array('c', ' ' * xbuffer))
        xcaret = ycaret = 0
        if wnd == GetFocus():
            SetCaretPos(xcaret * xchar, ycaret * ychar)
        return 0

    elif msg == WM_SETFOCUS:
        CreateCaret(wnd, 0, xchar, ychar)
        SetCaretPos(xcaret * xchar, ycaret * ychar)
        ShowCaret(wnd)
        return 0

    elif msg == WM_KILLFOCUS:
        HideCaret(wnd)
        DestroyCaret()
        return 0

    elif msg == WM_KEYDOWN:
        if wparam == VK_HOME:
            xcaret = 0
        elif wparam == VK_END:
            xcaret = xbuffer - 1
        elif wparam == VK_PRIOR:
            ycaret = 0
        elif wparam == VK_NEXT:
            ycaret = ybuffer - 1
        elif wparam == VK_LEFT:
            xcaret = max(xcaret - 1, 0)
        elif wparam == VK_RIGHT:
            xcaret = min(xcaret + 1, xbuffer - 1)
        elif wparam == VK_UP:
            ycaret = max(ycaret - 1, 0)
        elif wparam == VK_DOWN:
            ycaret = min(ycaret + 1, ybuffer - 1)
        elif wparam == VK_DELETE:
            lines[ycaret].pop(xcaret)
            lines[ycaret].insert(xbuffer - 1, ' ')
            HideCaret(wnd)
            dc = GetDC(wnd)
            SelectObject(dc, GetStockObject(SYSTEM_FIXED_FONT))
            TextOut(dc, xcaret * xchar, ycaret * ychar,
                    lines[ycaret][xcaret:], xbuffer - xcaret)
            ShowCaret(wnd)
            ReleaseDC(wnd, dc)
        SetCaretPos(xcaret * xchar, ycaret * ychar)
        return 0

    elif msg == WM_CHAR:
        for i in range(LOWORD(lparam)):
            if wparam == ord('\b'):
                if xcaret > 0:
                    xcaret -= 1
                    SendMessage(wnd, WM_KEYDOWN, VK_DELETE, 1)
            elif wparam == ord('\t'):
                while 1:
                    SendMessage(wnd, WM_CHAR, ord(' '), 1)
                    if xcaret % 8 == 0:
                        break
            elif wparam == ord('\n'):
                ycaret += 1
                if ycaret == ybuffer:
                    ycaret = 0
            elif wparam == ord('\r'):
                xcaret = 0
                ycaret += 1
                if ycaret == ybuffer:
                    ycaret = 0
            elif wparam == 0x1b:
                lines = []
                for i in range(ybuffer):
                    lines.append(array.array('c', ' ' * xbuffer))
                xcaret = ycaret = 0
                InvalidateRect(wnd, None, FALSE)
            else:
                lines[ycaret][xcaret] = chr(wparam)
                HideCaret(wnd)
                dc = GetDC(wnd)
                SelectObject(dc, GetStockObject(SYSTEM_FIXED_FONT))
                TextOut(dc, xcaret * xchar, ycaret * ychar,
                        lines[ycaret][xcaret], 1)
                ShowCaret(wnd)
                ReleaseDC(wnd, dc)
                xcaret += 1
                if xcaret == xbuffer:
                    xcaret = 0
                    ycaret += 1
                    if ycaret == ybuffer:
                        ycaret = 0
        SetCaretPos(xcaret * xchar, ycaret * ychar)
        return 0

    elif msg == WM_PAINT:
        dc = BeginPaint(wnd, ps)
        SelectObject(dc, GetStockObject(SYSTEM_FIXED_FONT))
        for y in range(ybuffer):
            TextOut(dc, 0, y * ychar, lines[y].tostring(), xbuffer)
        EndPaint(wnd, ps)
        return 0

    elif msg == WM_DESTROY:
        PostQuitMessage(0)
        return 0

    return DefWindowProc(wnd, msg, wparam, lparam)

def winmain(inst, prev_inst, cmd_line, cmd_show):
    app_name = 'Type'

    wc = WNDCLASSEX()
    wc.style = CS_HREDRAW | CS_VREDRAW
    wc.wnd_proc = wnd_proc
    wc.inst = inst
    wc.cursor = LoadCursor(0, IDC_ARROW)
    wc.background = GetStockObject(WHITE_BRUSH)
    wc.name = app_name
    RegisterClassEx(wc)

    wnd = CreateWindow(app_name, 'Typing Program', WS_OVERLAPPEDWINDOW,
                       CW_USEDEFAULT, CW_USEDEFAULT,
                       CW_USEDEFAULT, CW_USEDEFAULT, 0, 0, inst, 0)

    ShowWindow(wnd, cmd_show)
    UpdateWindow(wnd)

    msg = MSG()
    while 1:
        if not GetMessage(msg, 0, 0, 0):
            break
        TranslateMessage(msg)
        DispatchMessage(msg)
    return msg.wparam
