__version__ = '0.01'

import string

class Rec:
    def __init__(self, type):
        self.rec_type = type
    def __repr__(self):
        parts = []
        parts.append('(')
        parts.append(self.rec_type)
        parts.append(': ')
        num = 0
        for name in dir(self):
            if name == 'rec_type':
                continue
            if num:
                parts.append(', ')
            num = num + 1
            parts.append('%s=%s' % (name, getattr(self, name)))
        parts.append(')')
        return string.join(parts, '')

%%
parser Sdk32:
    ignore:     r'\s+'
    ignore:     r'#.*'
    token CODE: r'\{\{.*\}\}'
    token ID:   r'[a-zA-Z][a-zA-Z0-9_]*'
    token STR:  r'"([^\\"]+|\\.)*"'
    token COMMENT: r'--.*--'
    token END:  r'$'

    rule module:
        "module:" ID ";"             {{ m = Rec('module') }}
                                     {{ m.id = ID }}
                                     {{ m.type_list = [] }}
                                     {{ m.const_list = [] }}
                                     {{ m.callback_list = [] }}
                                     {{ m.struct_list = [] }}
                                     {{ m.func_list = [] }}
                                     {{ m.header_code = [] }}
                                     {{ m.init_code = [] }}
        ( "types:" type_list           {{ m.type_list.extend(type_list) }}
        | "const:" const_list          {{ m.const_list.extend(const_list) }}
        | "callbacks:" callback_list   {{ m.callback_list.extend(callback_list) }}
        | "structs:" struct_list       {{ m.struct_list.extend(struct_list) }}
        | "funcs:" func_list           {{ m.func_list.extend(func_list) }}
        | "header:" code_list          {{ m.header_code.extend(code_list) }}
        | "init:" code_list            {{ m.init_code.extend(code_list) }}
        )* END                       {{ return m }}

    rule type_list:
        type                         {{ l = [type] }}
        ( type                         {{ l.append(type) }}
        )*                           {{ return l }}
    rule type:
        ID                           {{ t = Rec('type') }}
                                     {{ t.name = ID }}
        "=" ID                       {{ t.equiv = ID }}
        (                              {{ t.ptr = 0 }}
        | "\\*"                        {{ t.ptr = 1 }}
        )  ";"
                                     {{ return t }}

    rule const_list:
        const                        {{ l = [const] }}
        ( const                        {{ l.append(const) }}
        )*                           {{ return l }}
    rule const:
        ID                           {{ c = Rec('const') }}
                                     {{ c.name = ID }}
        "\\(" ID                     {{ c.group = ID }}
        (                              {{ c.type = None }}
        | "," ID                       {{ c.type = ID }}
        )  "\\)" "=" value_list ";"  {{ c.value_list = value_list }}
                                     {{ return c }}
    rule value_list:
        ID                           {{ l = [ID] }}
        ( "," ID                       {{ l.append(ID) }}
        )*                           {{ return l }}

    rule callback_list:
        callback                     {{ l = [callback] }}
        ( callback                     {{ l.append(callback) }}
        )*                           {{ return l }}
    rule callback:
        ID                           {{ c = Rec('callback') }}
                                     {{ c.name = ID }}
        "=" ID                       {{ c.ret = ID }}
        ( "\\(\\);"                    {{ c.arg_list = [] }}
        | "\\(" arg_list "\\);"        {{ c.arg_list = arg_list }}
        )                            {{ return c }}

    rule struct_list:
        struct                       {{ l = [struct] }}
        ( struct                       {{ l.append(struct) }}
        )*                           {{ return l }}
    rule struct:
        ID                           {{ s = Rec('struct') }}
                                     {{ s.name = ID }}
        "=" ID "{" elem_list "}"     {{ s.tag = ID }}
                                     {{ s.elem_list = elem_list }}
                                     {{ return s }}

    rule elem_list:
        elem                         {{ l = [elem] }}
        ( elem                         {{ l.append(elem) }}
        )*                           {{ return l }}
    rule elem:
        ID                           {{ e = Rec('elem') }}
                                     {{ e.type = ID }}
        ID                           {{ e.name = ID }}
        (                              {{ e.rename = e.name }}
        | "\\(" ID "\\)"               {{ e.rename = ID }}
        )
        ( "\\[" attr_list "\\]"        {{ e.attr_list = attr_list }}
        |                              {{ e.attr_list = [] }}
        ) ";"                        {{ return e }}

    rule func_list:
        func                         {{ l = [func] }}
        ( func                         {{ l.append(func) }}
        )*                           {{ return l }}
    rule func:
        ID                           {{ f = Rec('func') }}
                                     {{ f.ret = ID }}
        ID                           {{ f.name = ID }}
        ( "\\(\\);"                    {{ f.arg_list = [] }}
        | "\\(" arg_list "\\);"        {{ f.arg_list = arg_list }}
        )
        ( code_list                    {{ f.code_list = code_list }}
        |                              {{ f.code_list = [] }}
        )                            {{ return f }}
    rule arg_list:
        arg                          {{ l = [arg] }}
        ( "," arg                      {{ l.append(arg) }}
        )*                           {{ return l }}
    rule arg:
        ID                           {{ a = Rec('arg') }}
                                     {{ a.type = ID }}
        ( "\\*"                        {{ a.ptr = 1 }}
        |                              {{ a.ptr = 0 }}
        )
        ID                           {{ a.name = ID }}
        ( "\\[" attr_list "\\]"        {{ a.attr_list = attr_list }}
        |                              {{ a.attr_list = [] }}
        )
        ( COMMENT                      {{ a.comment = string.strip(COMMENT[2:-2]) }}
        |                              {{ a.comment = '' }}
        )                            {{ return a }}

    rule attr_list:
        attr                         {{ l = [attr] }}
        ( "," attr                     {{ l.append(attr) }}
        )*                           {{ return l }}
    rule attr:
        ID                           {{ o = Rec('attr') }}
                                     {{ o.name = ID }}
        ( "="
          ( ID                         {{ o.value = ID }}
          | STR                        {{ o.value = eval(STR) }}
          )
        |                              {{ o.value = None }}
        )                            {{ return o }}

    rule code_list:
        CODE                         {{ l = [CODE[2:-2]] }}
        ( CODE                         {{ l.append(CODE[2:-2]) }}
        )*                           {{ return l }}
